MODULE I2CHygro;
(* =========================================================================  
   I2C Humidity and Temperature Measurement Program
   
   Description:
     Writes the current humidity and temperature to output
      
   Target: 
     Embedded Project Oberon with a TI HDC1080 Digital Humidity and
     Temperature Sensor connected to the I2C bus

   (c) 2021 CFB Software   
   https://www.astrobe.com  

  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)
   
IMPORT I2C, Out, Oberon;

PROCEDURE Delay(ms: INTEGER);
VAR
  startTime: INTEGER;
BEGIN
  startTime := Oberon.Time();
  WHILE Oberon.Time() - startTime < ms DO END
END Delay;


PROCEDURE DataToDegrees(data: ARRAY OF BYTE): REAL;
VAR
  value: INTEGER;
BEGIN
  value := data[0] * 256 + data[1];
  RETURN ((FLT(value) * 165.0) / 65536.0) - 40.0 
END DataToDegrees;


PROCEDURE DataToHumidity(data: ARRAY OF BYTE): REAL;
VAR
  value: INTEGER;
BEGIN
  value := data[0] * 256 + data[1];
  RETURN (FLT(value) / 65536.0) * 100.0
END DataToHumidity;


PROCEDURE DisplayResults(title: ARRAY OF CHAR; value: REAL; units: ARRAY OF CHAR);
(* Output with single digit after the decimal point *)
VAR
  val10: INTEGER;
BEGIN
  Out.String(title);
  (* Scale and round *)
  val10 := FLOOR((value * 10.0) + 5.0);
  Out.Int(val10 DIV 10, 0); Out.Char("."); Out.Int(val10 MOD 10, 0);
  Out.String(units); Out.Ln()
END DisplayResults;


PROCEDURE Show*;
CONST 
  HygroAddr = 040H;
  RegTemperature = 00H;
  RegHumidity = 01H;
  freq = 100000;
VAR
  data: ARRAY 2 OF BYTE;
  temperature, humidity: REAL; 
BEGIN
  Out.String("I2CHygro"); Out.Ln;
  I2C.Init(freq);
  Delay(100);
  
  I2C.WriteByte(HygroAddr, RegHumidity); 
  Delay(15);
  I2C.Read(HygroAddr, data);
  humidity := DataToHumidity(data);

  I2C.WriteByte(HygroAddr, RegTemperature); 
  Delay(15);
  I2C.Read(HygroAddr, data);
  temperature := DataToDegrees(data);

  DisplayResults("Humidity   : ", humidity, "%");
  DisplayResults("Temperature: ", temperature, " deg C")

END Show;

END I2CHygro.

  
  
  
    
  


